/*
	Emailscanner - The program filters email addresses from a file.
	Copyright 2020 Marek Küthe
	
	This program is free software. It comes without any warranty, to
	the extent permitted by applicable law. You can redistribute it
	and/or modify it under the terms of the Do What The Fuck You Want
	To Public License, Version 2, as published by Sam Hocevar. See
	http://www.wtfpl.net/ for more details.
*/

package emailscanner;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedList;
import java.util.zip.ZipFile;

/**
 * This class gets a file and reads its content. It works txt, html and odt files. With odt files, the zip entries content.xml and meta.xml are read because they could hide the e-mail address.
 */
public class Extractor {

	public enum EXT_TYPE { HTML, TXT, ODT }
	
	protected File mainFile;
	protected LinkedList<String> res;
	
	{
		mainFile = null;
		res = new LinkedList<String>();
	}
	
	public Extractor(File file) {
		mainFile = file;
	}
	
	public Extractor(String file) {
		mainFile = new File(file);
	}
	
	/**
	 * Reads from an InputStreamReader and adds each line to the global LinkedList res.
	 * @param stream The stream from which the lines should be read. Later in the function, the stream is sent to a BufferedReader.
	 */
	protected void ReadAndAdd(InputStreamReader stream) throws IOException {
		BufferedReader contentReader = new BufferedReader(stream);
		
		while(true) {
			String str = contentReader.readLine();
			if(str == null)
				break;
			res.add(str);
		}
		
		contentReader.close();
	}
	
	/**
	 * Extracts the data from a file. With txt and html files it is actually a simple call to the ReadAndAdd function. With an odt file, however, it is first necessary to open it as a ZIP file.
	 * @param type The type. It can be TXT, HTML, or ODT. The types are available in UNUM EXT_TYPE.
	 */
	public void extract(EXT_TYPE type) throws RuntimeException, IOException {
		if(! (mainFile.exists() && mainFile.canRead())) {
			throw new RuntimeException("The file does not exist or cannot be read.");
		}

		switch(type) {
		case HTML:
		case TXT:
		{
			FileReader fileReader = new FileReader(mainFile);
			ReadAndAdd(fileReader);
			fileReader.close();
			break;
		}  /* end of case html, txt */
		case ODT:
		{
			ZipFile zipfile = new ZipFile(mainFile);

			InputStreamReader content_xml = new InputStreamReader(zipfile.getInputStream(zipfile.getEntry("content.xml")));
			InputStreamReader meta_xml = new InputStreamReader(zipfile.getInputStream(zipfile.getEntry("meta.xml")));
			
			ReadAndAdd(content_xml);
			ReadAndAdd(meta_xml);
			
			zipfile.close();
			break;
		}  /* end of case odt */
		}
	}
	
	/**
	 * Reads a line on the global LinkedList res and returns it. It is read from the bottom up. This can happen because the e-mail address is searched line by line and it does not matter in which order the lines are read.
	 * @return The corresponding line
	 */
	public String getLine() {
		if(res.isEmpty())
			return null;
		else
			return res.pop();
	}
	
}
